@@ -15,57 +15,66 @@ from itertools import combinations
 import shutil
 
 LINES = '-' * 80
+CONST_COLOR_SHORTCUTS = {
+    'b': 'blue',
+    'g': 'green',
+    'r': 'red',
+    'c': 'cyan',
+    'm': 'magenta',
+    'y': 'yellow',
+    'k': 'black',
+    'w': 'white',
+    'bo': 'bold',
+    'un': 'underlined',
+    're': 'reverse',
+}
+
 CONST_COLOR_MAP = {
-    'ResetAll': "\033[0m",
-    'Bold': "\033[1m",
-    'Dim': "\033[2m",
-    'Underlined ': "\033[4m",
-    'Blink      ': "\033[5m",
-    'Reverse    ': "\033[7m",
-    'Hidden     ': "\033[8m",
+    'reset_all': "\033[0m",
+    'bold': "\033[1m",
+    'underlined': "\033[4m",
+    'reverse': "\033[7m",  # switch font color and background color
 
-    'ResetBold': "\033[21m",
-    'ResetDim': "\033[22m",
-    'ResetUnderlined': "\033[24m",
-    'ResetBlink': "\033[25m",
-    'ResetReverse': "\033[27m",
-    'ResetHidden': "\033[28m",
+    'blue': "\033[34m",
+    'background_blue': "\033[44m",
+    'light_blue': "\033[94m",
+    'background_light_blue': "\033[104m",
 
-    'Default': "\033[39m",
-    'Black': "\033[97m",
-    'Red': "\033[31m",
-    'Green': "\033[32m",
-    'Yellow': "\033[33m",
-    'Blue': "\033[34m",
-    'Magenta': "\033[35m",
-    'Cyan': "\033[36m",
-    'LightGray': "\033[37m",
-    'DarkGray': "\033[90m",
-    'LightRed': "\033[91m",
-    'LightGreen': "\033[92m",
-    'LightYellow': "\033[93m",
-    'LightBlue': "\033[94m",
-    'LightMagenta': "\033[95m",
-    'LightCyan': "\033[96m",
-    'White': "\033[30m",
+    'green': "\033[32m",
+    'background_green': "\033[42m",
+    'light_green': "\033[92m",
+    'background_light_green': "\033[102m",
 
-    'BackgroundDefault': "\033[49m",
-    'BackgroundBlack': "\033[107m",
-    'BackgroundRed': "\033[41m",
-    'BackgroundGreen': "\033[42m",
-    'BackgroundYellow': "\033[43m",
-    'BackgroundBlue': "\033[44m",
-    'BackgroundMagenta': "\033[45m",
-    'BackgroundCyan': "\033[46m",
-    'BackgroundLightGray': "\033[47m",
-    'BackgroundDarkGray': "\033[100m",
-    'BackgroundLightRed': "\033[101m",
-    'BackgroundLightGreen': "\033[102m",
-    'BackgroundLightYellow': "\033[103m",
-    'BackgroundLightBlue': "\033[104m",
-    'BackgroundLightMagenta': "\033[105m",
-    'BackgroundLightCyan': "\033[106m",
-    'BackgroundWhite': "\033[40m"
+    'red': "\033[31m",
+    'background_red': "\033[41m",
+    'light_red': "\033[91m",
+    'background_light_red': "\033[101m",
+
+    'cyan': "\033[36m",
+    'background_cyan': "\033[46m",
+    'background_light_cyan': "\033[106m",
+    'light_cyan': "\033[96m",
+
+    'magenta': "\033[35m",
+    'background_magenta': "\033[45m",
+    'light_magenta': "\033[95m",
+    'background_light_magenta': "\033[105m",
+
+    'yellow': "\033[33m",
+    'background_yellow': "\033[43m",
+    'light_yellow': "\033[93m",
+    'background_light_yellow': "\033[103m",
+
+    'light_gray': "\033[37m",
+    'background_light_gray': "\033[47m",
+    'dark_gray': "\033[90m",
+    'background_dark_gray': "\033[100m",
+
+    'black': "\033[97m",
+    'background_black': "\033[107m",
+
+    'white': "\033[30m",
+    'background_white': "\033[40m",
 }
 
 
@@ -77,7 +86,7 @@ def exception_error(e, tabs: int = 1):
     :return:
     """
     error_str = '{}{}: {}'.format(tabs * '\t', get_function_name(depth=2), e)
-    print(add_color(string=error_str, color='Red'))
+    print(add_color(string=error_str, op1='Red'))
     return
 
 
@@ -199,8 +208,9 @@ def get_cuda_version(ack: bool = False, tabs: int = 1) -> str:
     :return: cuda version if found on environment variables
     see get_cuda_version_test()
     """
-    if 'CUDA_PATH' in os.environ:
-        cuda_v = '* CUDA Version: {}'.format(os.path.basename(os.environ['CUDA_PATH']))
+    cuda_path = get_env_variable(key='CUDA_PATH')
+    if cuda_path is not None:
+        cuda_v = '* CUDA Version: {}'.format(os.path.basename(cuda_path))
     else:
         cuda_v = '* No CUDA_PATH found'
     if ack:
@@ -213,7 +223,6 @@ def get_env_variables(ack: bool = False, tabs: int = 1) -> dict:
     :param ack:
     :param tabs:
     :return: dict with envs
-    prints env variables
     see get_env_variables_test()
     """
     env_d = dict(os.environ)
@@ -224,6 +233,58 @@ def get_env_variables(ack: bool = False, tabs: int = 1) -> dict:
     return env_d
 
 
+def set_env_variable(key: str, val: str, ack: bool = False, tabs: int = 1) -> None:
+    """
+    :param key:
+    :param val:
+    :param ack:
+    :param tabs:
+    insert new env variable
+    see set_env_variable_test()
+    """
+    key = key.upper()
+    os.environ[key] = val
+    if ack:
+        print('{}Inserted to environment: {} = {}'.format(tabs * '\t', key, val))
+    return
+
+
+def get_env_variable(key: str, ack: bool = False, tabs: int = 1) -> str:
+    """
+    :param key:
+    :param ack:
+    :param tabs:
+    :return: env variable value
+    see get_env_variable_test()
+    """
+    key = key.upper()
+    ret_val = os.environ[key] if key in os.environ else None
+    if ack:
+        if ret_val is not None:
+            print('{}{} = {}'.format(tabs * '\t', key, ret_val))
+        else:
+            print('{}{} Not found'.format(tabs * '\t', key))
+    return ret_val
+
+
+def del_env_variable(key: str, ack: bool = False, tabs: int = 1) -> None:
+    """
+    :param key:
+    :param ack:
+    :param tabs:
+    :return: env variable value
+    see get_env_variable_test()
+    """
+    key = key.upper()
+    status = 'Not found'
+    if key in os.environ:
+        del os.environ[key]
+        status = 'Deleted'
+    if ack:
+        print('{}{} {}'.format(tabs * '\t', key, status))
+    return
+
+
 def make_cuda_invisible() -> None:
     """
         disable gpu 0
@@ -236,7 +297,7 @@ def make_cuda_invisible() -> None:
         read more: https://docs.nvidia.com/cuda/cuda-c-programming-guide/index.html#env-vars
         see make_cuda_invisible_test()
     """
-    os.environ['CUDA_VISIBLE_DEVICES'] = '-1, 0'
+    set_env_variable(key='CUDA_VISIBLE_DEVICES', val='-1, 0')
     return
 
 
@@ -289,6 +350,7 @@ def get_tensorflow_version(ack: bool = False, tabs: int = 1) -> str:
     see get_tensorflow_version_test()
     """
     try:
+        # noinspection PyUnresolvedReferences,PyPackageRequirements
         import tensorflow as tf
         string = '* TensorFlow Version {}'.format(tf.__version__)
         if ack:
@@ -301,6 +363,7 @@ def get_tensorflow_version(ack: bool = False, tabs: int = 1) -> str:
 def main_wrapper(
         main_function: Callable,
         seed: int = -1,
+        ipv4: bool = False,
         cuda_off: bool = False,
         torch_v: bool = False,
         tf_v: bool = False,
@@ -310,6 +373,7 @@ def main_wrapper(
     """
     :param main_function: the function to run
     :param seed: if -1 no seed, else set_seed(seed=seed)
+    :param ipv4: print computer ipv4
     :param cuda_off: make gpu invisible and force run on cpu
     :param torch_v: print torch version
     :param tf_v: print tensorflow version
@@ -324,9 +388,19 @@ def main_wrapper(
     print('main_wrapper:')
     print('* Run started at {}'.format(get_current_date_hour()))
     print('* Python Version {}'.format(sys.version))
+    print('* Interpreter: {}'.format(sys.executable))
     print('* Working Dir: {}'.format(os.getcwd()))
     print('* Computer Name: {}'.format(get_pc_name()))
     print('* Computer Mac: {}'.format(get_mac_address()))
+
+    if ipv4:
+        try:
+            from wizzi_utils.socket.socket_tools import get_ipv4
+            print('* Computer ipv4: {}'.format(get_ipv4()))
+        except (ImportError, ModuleNotFoundError, NameError) as err:
+            string = '* {}'.format(err)
+            print(string)
+
     cuda_msg = get_cuda_version()
     if cuda_off:
         make_cuda_invisible()
@@ -339,20 +413,20 @@ def main_wrapper(
             print(get_torch_version())
         except (ImportError, ModuleNotFoundError, NameError) as err:
             string = '* {}'.format(err)
-            exception_error(string)
+            print(string)
     if tf_v:
         try:
             print(get_tensorflow_version())
         except (ImportError, ModuleNotFoundError, NameError) as err:
             string = '* {}'.format(err)
-            exception_error(string)
+            print(string)
     if cv2_v:
         try:
             from wizzi_utils.open_cv.open_cv_tools import get_cv_version
             print(get_cv_version())
         except (ImportError, ModuleNotFoundError, NameError) as err:
             string = '* {}'.format(err)
-            exception_error(string)
+            print(string)
 
     if seed > -1:
         set_seed(seed=seed)
@@ -517,6 +591,7 @@ def load_npz(path: str, ack: bool = True, tabs: int = 1) -> dict:
     """
     arrays_obj = np.load(path)
     arrays_dict = {}
+    # noinspection PyUnresolvedReferences
     for k in arrays_obj.files:
         arrays_dict[k] = arrays_obj[k]
     if ack:
@@ -811,25 +886,56 @@ def get_base_file_and_function_name(depth: int = 1, ack: bool = False, tabs: int
     return ret_val
 
 
-def add_color(string: str, color: str = 'Red') -> str:
+def add_color(
+        string: str,
+        op1: str = 'red',
+        extra_ops: list = None,
+) -> str:
     """
     :param string:
-    :param color: from color map
-    # todo add option for combination (like bold, underlined ...)
-    # todo change to lower case
-    # todo color numbers maybe wrong
+    :param op1: color, bg_color, bold, underline, reverse
+    :param extra_ops: color, bg_color, bold, underline, reverse
+    colors:
+    * X = { blue(b), green(g), red(r), cyan(c), magenta(m), yellow(y) }
+        for x in X: x, background_x, light_x, background_light_x
+    * more colors:
+        light_gray, background_light_gray, dark_gray, background_dark_gray
+        black, background_black
+        white, background_white
+
+    special options:
+    * bold(bo), underlined(un), reverse(re)
+
     to see all colors and options:
-        for k, v in CONST_COLOR_MAP.items():
-            print('{}{}{}'.format(v, k, CONST_COLOR_MAP['ResetAll']))
+        for k, v in wu.CONST_COLOR_MAP.items():
+            print('{}{}{}'.format(v, k, wu.CONST_COLOR_MAP['reset_all']))
     see add_color_test()
     """
+    out_string = string
+    # first to lower and check shortcuts
+    op1 = op1.lower()
+    if op1 in CONST_COLOR_SHORTCUTS:
+        op1 = CONST_COLOR_SHORTCUTS[op1]
 
-    if color in CONST_COLOR_MAP:
+    if extra_ops is not None:
+        for i in range(len(extra_ops)):
+            extra_ops[i] = extra_ops[i].lower()
+            if extra_ops[i] in CONST_COLOR_SHORTCUTS:
+                extra_ops[i] = CONST_COLOR_SHORTCUTS[extra_ops[i]]
+
+    # second build str with color tags
+    if op1 in CONST_COLOR_MAP:
         # concat (color, string, reset tag)
-        string = '{}{}{}'.format(CONST_COLOR_MAP[color], string, CONST_COLOR_MAP['ResetAll'])
-    return string
+        out_string = CONST_COLOR_MAP[op1]
+        if extra_ops is not None:
+            for op in extra_ops:
+                if op in CONST_COLOR_MAP:
+                    out_string += CONST_COLOR_MAP[op]
+        out_string += '{}{}'.format(string, CONST_COLOR_MAP['reset_all'])
+    return out_string
 
 
+# noinspection PyUnresolvedReferences
 def init_logger(logger_path: str = './log.txt') -> io.TextIOWrapper:
     """
     :param: logger_path
